//
//  internet.c
//

// Retrieve files from the web via HTTP

/* This is the text that appears when a user requests help about this program
<help>

 -- internet --

view files from the web.

Usage:
  internet <URL>

This command will send HTTP commands to view documents and elements
referenced by them - for example images.

</help>
*/


#include <errno.h>
#include <libgen.h>
#include <libintl.h>
#include <locale.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/api.h>
#include <sys/env.h>
#include <sys/file.h>
#include <sys/http.h>
#include <sys/url.h>
#include <sys/vis.h>
#include <sys/vsh.h>
#include <sys/font.h>
#include <sys/network.h>
#include <sys/paths.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/window.h>
#include <arpa/inet.h>
#include <netinet/in.h>

#define DEFAULT_RECURSIONS	16

#define _(string) gettext(string)
#define WINDOW_TITLE	_("Internet Browser v0.1")

extern int debugLibHttp;

static int graphics = 0;
static objectKey window = NULL;
static objectKey urlField = NULL;
static objectKey goButton = NULL;
static objectKey textArea = NULL;
static char *remoteHost = NULL;

__attribute__((format(printf, 1, 2)))
static void error(const char *format, ...)
{
	// Generic error message code for either text or graphics modes

	va_list list;
	char output[MAXSTRINGLENGTH + 1];

	va_start(list, format);
	vsnprintf(output, MAXSTRINGLENGTH, format, list);
	va_end(list);

	if (graphics)
		windowNewErrorDialog(window, _("Error"), output);
	else
		printf("\n%s\n", output);
}

__attribute__((noreturn))
static void quit(int status)
{
	// To terminate the signal handler
	signal(0, SIG_DFL);

	if (graphics)
	{
		windowGuiStop();

		if (window)
			windowDestroy(window);
	}

	exit(status);
}


static void interrupt(int sig)
{
	// This is our interrupt signal handler.
	if (sig == SIGINT){}
		
}

static void usage(char *name)
{
	fprintf(stderr, _("usage:\n%s <URL>\n"), name);
	return;
}


static int getUrl(const char *urlString, int maxRecursions)
{
	int status = 0;
	urlInfo *url = NULL;
	// char fullPath[MAX_PATH_NAME_LENGTH + 1];
	// char *dirName = NULL;
	variableList header;
	const char *statusCode = NULL;
	const char *redirect = NULL;
	char *content = NULL;
	int contentLen = 0;
	// fileStream outStream;

	printf("Parsing '%s'\n", urlString);

	status = httpParseUrl(urlString, &url);
	if (status < 0)
	{
		fprintf(stderr, "%s\n", _("Error parsing URL"));
		goto out;
	}

	// Compose the full path to the file we'll be saving
	// strncpy(fullPath, urlString, MAX_PATH_NAME_LENGTH);
	// if (url->host && url->path)
	// {
	// 	snprintf(fullPath, MAX_PATH_NAME_LENGTH, "%s/%s", url->host,
	// 		url->path);
	// }

	// Get the directory part and try to make sure it exists
	// dirName = dirname(fullPath);
	// if (dirName)
	// {
	// 	vshMakeDirRecursive(dirName);
	// 	free(dirName);
	// }

	status = httpGet(url, &header, &content, &contentLen,
		10000 /* 10 second timeout */);
	if (status < 0)
	{
		statusCode = variableListGet(&header, HTTP_STATUSCODE_VAR);
		if (statusCode)
		{
			switch (atoi(statusCode))
			{
				case HTTP_STATUSCODE_MOVEDPERMANENTLY:
				case HTTP_STATUSCODE_SEEOTHER:
				case HTTP_STATUSCODE_TEMPORARYREDIRECT:
				case HTTP_STATUSCODE_PERMANENTREDIRECT:
				{
					// See whether we can retrieve it at another URL
					redirect = variableListGet(&header, HTTP_HEADER_LOCATION);
					if (redirect)
					{
						if (!strcmp(urlString, redirect))
						{
							fprintf(stderr, "%s %s\n", _("Error redirection "
								"to same URL"), urlString);
							status = ERR_ALREADY;
							goto out;
						}

						status = getUrl(redirect, (maxRecursions - 1));
						goto out;
					}
				}
			}
		}

		fprintf(stderr, "%s %s\n", _("Error retrieving"), urlString);
		goto out;
	}

	// status = fileStreamOpen(fullPath, (OPENMODE_WRITE | OPENMODE_CREATE |
	// 	OPENMODE_TRUNCATE), &outStream);
	// if (status < 0)
	// {
	// 	fprintf(stderr, "%s %s\n", _("Error writing"), urlString);
	// 	goto out;
	// }

	// status = fileStreamWrite(&outStream, contentLen, content);

	// fileStreamClose(&outStream);

	// if (status < 0)
	// {
	// 	fprintf(stderr, "%s %s\n", _("Error writing"), urlString);
	// 	goto out;
	// }
	printf("%s",content);
	status = 0;

out:
	if (content)
		free(content);
	if (header.memory)
		variableListDestroy(&header);
	if (url)
		httpFreeUrlInfo(url);

	return (status);
}
static void refreshWindow(void)
{
	// We got a 'window refresh' event (probably because of a language
	// switch), so we need to update things

	// Re-get the language setting
	setlocale(LC_ALL, getenv(ENV_LANG));
	textdomain("wget");

	// Re-get the character set
	if (getenv(ENV_CHARSET))
		windowSetCharSet(window, getenv(ENV_CHARSET));

	// Refresh the window title
	windowSetTitle(window, WINDOW_TITLE);

	// Re-layout the window (not necessary if no components have changed)
	windowLayout(window);
}

static void eventHandler(objectKey key, windowEvent *event)
{
	// Check for window events.
	if (key == window)
	{
		// Check for window refresh
		if (event->type == WINDOW_EVENT_WINDOW_REFRESH)
			refreshWindow();

		// Check for the window being closed
		else if (event->type == WINDOW_EVENT_WINDOW_CLOSE)
			quit(0);
	}
	else if ((key == goButton) && (event->type ==
		WINDOW_EVENT_MOUSE_LEFTUP))
	{
		windowComponentGetData(urlField, remoteHost, 1000);
		getUrl(remoteHost, DEFAULT_RECURSIONS);
		refreshWindow();
	}
}

static void constructWindow(void)
{
	componentParameters params;

	// Create a new window
	if (window == NULL)
		window = windowNew(multitaskerGetCurrentProcessId(), WINDOW_TITLE);

	if (!window)
		return;

	memset(&params, 0, sizeof(componentParameters));

	// A text label to show whom we're pinging
	params.gridWidth = 1;
	params.gridHeight = 1;
	params.padLeft = 5;
	params.padRight = 5;
	params.padTop = 5;
	params.orientationX = orient_left;
	params.orientationY = orient_middle;
	if (urlField == NULL)
	{
		urlField = windowNewTextField(window, 28, &params);
		windowRegisterEventHandler(urlField, &eventHandler);
	}

	windowComponentSetData(urlField, remoteHost, 1000, 1 /* redraw */);

	params.font = fontGet(FONT_FAMILY_LIBMONO, FONT_STYLEFLAG_FIXED, 8, NULL);
	params.gridX = 30;
	if (goButton == NULL)
		goButton = windowNewButton(window, "->", NULL, &params);

	windowRegisterEventHandler(goButton, &eventHandler);
	windowComponentFocus(goButton);
	// Create a text area to show our ping activity
	params.gridX = 0;
	params.gridY = 1;
	if (textArea != NULL)
		windowComponentDestroy(textArea);
	textArea = windowNewTextArea(window, 100 /* columns */, 50 /* rows */,
		1000 /* bufferLines */, &params);
	windowSetTextOutput(textArea);
	textSetCursor(0);
	textInputSetEcho(0);

	// Create a 'Stop' button
	params.gridY = 2;
	params.padBottom = 5;
	params.orientationX = orient_center;
	params.flags |= COMP_PARAMS_FLAG_FIXEDWIDTH;
	params.font = NULL;
	

	// Register an event handler to catch window close events
	windowRegisterEventHandler(window, &eventHandler);

	windowSetVisible(window, 1);
}

int main(int argc, char *argv[])
{
	int status = 0;
	char opt;
	

	// Are graphics enabled?
	graphics = graphicsAreEnabled();

	setlocale(LC_ALL, getenv(ENV_LANG));
	textdomain("wget");

	// Make sure networking is enabled
	if (!networkEnabled())
	{
		fprintf(stderr, "%s\n", _("Networking is not currently enabled"));
		return (status = ERR_NOTINITIALIZED);
	}

	// Check options
	while (strchr("T?", (opt = getopt(argc, argv, "T"))))
	{
		switch (opt)
		{
			case 'T':
				// Force text mode
				graphics = 0;
				break;

			default:
				error(_("Unknown option '%c'"), optopt);
				usage(argv[0]);
				return (status = ERR_INVALID);
		}
	}

	if (argc >= 2)
		remoteHost = argv[argc - 1];

	// If we are in graphics mode, and no destination has been specified, we
	// will create an interactive window which prompts for it.
	if (graphics)
	{
		
		remoteHost = "neverssl.com";
		constructWindow();
		windowGuiThread();
		status = getUrl(remoteHost, DEFAULT_RECURSIONS);
	}
	else
	{
		if (argc < 2)
		{
			usage(argv[0]);
			return (status = ERR_ARGUMENTCOUNT);
		}
		// Set up the signal handler for catching CTRL-C interrupt
		if (signal(SIGINT, &interrupt) == SIG_ERR)
		{
			error("%s", _("Error setting signal handler"));
			quit(status = ERR_NOTINITIALIZED);
		}
		printf("%s\n", remoteHost);
		status = getUrl(argv[2], DEFAULT_RECURSIONS);
	}

	return (status);
}

